"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable jest/expect-expect */
var Entity_1 = require("./Entity");
describe('Entity normalization', function () {
    it('normalizes an entity', function () {
        var entityBuilder = Entity_1.entity().id('id').name('items');
        var schema = Entity_1.buildSchema(entityBuilder);
        expect(schema.normalize({ "id": 1 })).toEqual({
            "result": '1',
            "entities": {
                "items": { "1": { "id": 1 } }
            }
        });
    });
    it('allows for other entities to be added', function () {
        var userBuilder = Entity_1.entity().id('name').name('users');
        var userSchema = Entity_1.buildSchema(userBuilder);
        var postBuilder = Entity_1.entity()
            .id('id')
            .name('posts')
            .prop('author', 'users')
            .define('users', userSchema);
        var postSchema = Entity_1.buildSchema(postBuilder);
        var testPost = {
            "id": 1,
            "title": 'Test Post',
            "author": {
                "name": 'Jack'
            }
        };
        var output = postSchema.normalize(testPost);
        expect(output).toEqual({
            "result": '1',
            "entities": {
                "posts": {
                    "1": { "id": 1, "title": 'Test Post', "author": 'Jack' }
                },
                'users': {
                    'Jack': { "name": 'Jack' }
                }
            }
        });
    });
    it('allows for recursive schemas', function () {
        var testUser = {
            "name": 'Jack',
            "bestFriend": {
                "name": 'Jill'
            }
        };
        var userBuilder = Entity_1.entity()
            .id('name')
            .name('users')
            .prop('bestFriend', 'users');
        var userSchema = Entity_1.buildSchema(userBuilder);
        expect(userSchema.normalize(testUser)).toEqual({
            "result": 'Jack',
            "entities": {
                "users": {
                    'Jack': { "name": 'Jack', "bestFriend": 'Jill' },
                    'Jill': { "name": 'Jill' }
                }
            }
        });
    });
    it('allows for array attributes', function () {
        var testUser = {
            "name": 'Jack',
            "posts": [
                {
                    "id": 1,
                    "title": 'Test Post 1'
                },
                {
                    "id": 2,
                    "title": 'Test Post 2'
                }
            ]
        };
        var postBuilder = Entity_1.entity().id('id').name('posts');
        var postSchema = Entity_1.buildSchema(postBuilder);
        var userBuilder = Entity_1.entity()
            .id('name')
            .name('users')
            .prop('posts', 'posts')
            .define('posts', postSchema);
        var userSchema = Entity_1.buildSchema(userBuilder);
        var result = userSchema.normalize(testUser);
        expect(result).toEqual({
            "result": 'Jack',
            "entities": {
                "users": {
                    'Jack': { "name": 'Jack', "posts": ['1', '2'] },
                },
                "posts": {
                    "1": { "id": 1, "title": 'Test Post 1' },
                    "2": { "id": 2, "title": 'Test Post 2' }
                }
            }
        });
    });
    it('allows for empty entities', function () {
        var testUser = {
            "name": 'Jack'
        };
        var foodSchema = Entity_1.buildSchema(Entity_1.entity().id('name').name('foods'));
        var userSchema = Entity_1.buildSchema(Entity_1.entity().id('name').name('users')
            .prop('favoriteFood', 'foods')
            .define('foods', foodSchema));
        var entities = userSchema.normalize(testUser).entities;
        expect(entities.foods).toEqual({});
    });
    describe('idAttribute', function () {
        it('can compute an id', function () {
            var testBuilder = Entity_1.entity()
                .name('tests')
                .computeId(function (input) { return input.foo + input.bar; });
            var testSchema = Entity_1.buildSchema(testBuilder);
            var test = {
                "foo": 'baz',
                "bar": 'qux'
            };
            expect(testSchema.normalize(test)).toEqual({
                "result": 'bazqux',
                "entities": {
                    "tests": {
                        'bazqux': { "foo": 'baz', "bar": 'qux' }
                    }
                }
            });
        });
        it('can build the entity\'s ID from the parent object', function () {
            var testBuilder = Entity_1.entity()
                .name('tests')
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                .computeId(function (input, parent) { return ['string', 'number'].includes(typeof parent.id) ?
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                "" + parent.id + input.foo :
                input.foo; });
            var testSchema = Entity_1.buildSchema(testBuilder);
            var holderBuilder = Entity_1.entity()
                .name('holders')
                .id('id')
                .prop('a', 'tests')
                .prop('b', 'tests')
                .define('tests', testSchema);
            var holderSchema = Entity_1.buildSchema(holderBuilder);
            var test = {
                "foo": 'baz',
                "bar": 'qux'
            };
            var holder = {
                "id": 42,
                "a": test,
                "b": test
            };
            expect(holderSchema.normalize(holder)).toEqual({
                "result": '42',
                "entities": {
                    'holders': {
                        '42': {
                            "id": 42,
                            "a": '42baz',
                            "b": '42baz'
                        }
                    },
                    'tests': {
                        '42baz': { "foo": 'baz', "bar": 'qux' },
                    }
                }
            });
        });
        it('can normalize entity IDs based on their object key', function () {
            var testBuilder = Entity_1.entity()
                .name('tests')
                .computeId(function (input, _parent, key) { return key ? key + input.foo : input.foo; });
            var testSchema = Entity_1.buildSchema(testBuilder);
            var holderBuilder = Entity_1.entity()
                .name('holders')
                .id('id')
                .prop('a', 'tests')
                .prop('b', 'tests')
                .define('tests', testSchema);
            var holderSchema = Entity_1.buildSchema(holderBuilder);
            var test = {
                "foo": 'baz',
                "bar": 'qux'
            };
            var holder = {
                "id": 42,
                "a": test,
                "b": test
            };
            expect(holderSchema.normalize(holder)).toEqual({
                "result": '42',
                "entities": {
                    'holders': {
                        '42': {
                            "id": 42,
                            "a": 'abaz',
                            "b": 'bbaz'
                        }
                    },
                    'tests': {
                        'abaz': { "foo": 'baz', "bar": 'qux' },
                        'bbaz': { "foo": 'baz', "bar": 'qux' }
                    }
                }
            });
        });
    });
    describe('mergeStrategy', function () {
        it('defaults to plain object merging', function () { });
        it('can use a custom merging strategy', function () { });
    });
    describe('processStrategy', function () {
        it('can use a custom processing strategy', function () {
            var processFunction = function (i) { return ({
                "foo": "" + i.id
            }); };
            var iBuilder = Entity_1.processedEntity(processFunction)
                .id('foo')
                .name('i');
            var iSchema = Entity_1.buildSchema(iBuilder);
            var testInput = {
                "id": 42
            };
            expect(iSchema.normalize(testInput)).toEqual({
                "result": '42',
                "entities": {
                    "i": {
                        '42': { "foo": '42' }
                    }
                }
            });
        });
        it('can use information from the parent in the process strategy', function () {
            var processFunction = function (i, parent, key) { return ({
                "foo": "" + i.id,
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                "pid": parent.id ? parent.id : null,
                "pkey": key
            }); };
            var iBuilder = Entity_1.processedEntity(processFunction)
                .id('foo')
                .name('i');
            var iSchema = Entity_1.buildSchema(iBuilder);
            var pBuilder = Entity_1.entity()
                .id('id')
                .name('p')
                .prop('foo', 'i')
                .define('i', iSchema);
            var pSchema = Entity_1.buildSchema(pBuilder);
            var testInput = {
                "id": 'test',
                "foo": {
                    "id": 42
                }
            };
            expect(pSchema.normalize(testInput)).toEqual({
                "result": 'test',
                "entities": {
                    'p': {
                        'test': { "id": 'test', "foo": '42' }
                    },
                    'i': {
                        '42': { "foo": '42', "pid": 'test', "pkey": 'foo' }
                    }
                }
            });
        });
    });
});
describe('Entity denormalization', function () {
    it('denormalizes an entity', function () { });
    it('denormalizes deep entities', function () { });
    it('denormalizes to undefined for missing data', function () { });
    it('denormalizes deep entities with records', function () { });
    it('denormalizes recursive dependencies', function () { });
    it('can denormalize already partially denormalized data', function () { });
    it('denormalizes entities with referential equality', function () { });
    it('denormalizes with fallback strategy', function () { });
});
describe('Entity Builder', function () {
    it('can set an ID prop that exists on its entity type', function () {
        var idEntity = Entity_1.entity().id('id');
        var indexEntity = Entity_1.entity().id('index');
        var testEntity = {
            "id": 'foo',
            "index": 42
        };
        expect(idEntity.idFunction(testEntity, null, undefined)).toBe('foo');
        expect(indexEntity.idFunction(testEntity, null, undefined)).toBe('42');
    });
    it('can set any name for the entity', function () {
        var fooEntity = Entity_1.entity().name('foo');
        var jarbledEntity = Entity_1.entity().name('/$%*(&@^\\');
        expect(fooEntity.nameProp).toBe('foo');
        expect(jarbledEntity.nameProp).toBe('/$%*(&@^\\');
    });
    it('can set a name twice, overriding the original', function () {
        var entityBuilder = Entity_1.entity().id('id').name('foo').name('bar');
        var entitySchema = Entity_1.buildSchema(entityBuilder);
        expect(entitySchema.nameProp).toBe('bar');
    });
    it('can set name and ID in either order', function () {
        var nameFirst = Entity_1.entity().name('foo').id('id');
        var idFirst = Entity_1.entity().id('index').name('bar');
        var testEntity = {
            "id": 'foo',
            "index": 42
        };
        expect(nameFirst.idFunction(testEntity, null, undefined)).toBe('foo');
        expect(nameFirst.nameProp).toBe('foo');
        expect(idFirst.idFunction(testEntity, null, undefined)).toBe('42');
        expect(idFirst.nameProp).toBe('bar');
    });
    it('can build from a complete builder', function () {
        var entityBuilder = Entity_1.entity().id('id').name('foo');
        var schema = Entity_1.buildSchema(entityBuilder);
        // eslint-disable-next-line @typescript-eslint/unbound-method
        expect(schema.normalize).toBeDefined();
    });
    it('cannot build from an incomplete builder', function () {
        var noInfo = Entity_1.entity();
        var idOnly = Entity_1.entity().id('id');
        var missingSchemas = Entity_1.entity().id('id').name('entities').prop('index', 'undefined');
        // @ts-expect-error Cannot build from an incomplete builder
        Entity_1.buildSchema(noInfo);
        // @ts-expect-error Cannot build from an incomplete builder
        Entity_1.buildSchema(idOnly);
        // @ts-expect-error Cannot build from an incomplete builder
        Entity_1.buildSchema(missingSchemas);
    });
});
/* eslint-enable @typescript-eslint/no-empty-function */
/* eslint-enable jest/expect-expect */
